module Pkgman
  class Container

    def initialize(image, version)
      @image = Docker::Image.create('fromImage' => "#{image}:#{version}")
      @container = Docker::Container.create('Image' => @image.id, 'Tty' => true)
      @container.start
    end

    def execute(command, cwd = '/root')
      puts "Executing #{command} in #{cwd}"

      options = {}
      options['WorkingDir'] = cwd if cwd

      command = ['/bin/bash', '-c', command]

      result = @container.exec(command, options) do |stream, chunk|
        puts "#{stream}: #{chunk}"
      end

      if result[2] > 0
        self.destroy
        raise Exception.new('Execution failed')
      end

      result
    end

    def download(what, dst)
      pth = File.join(dst, "export.rpm.tar")
      File.open(pth, 'w') do |file|
        @container.copy(what) do |chunk|
          file.write(chunk)
        end
      end

      Gem::Package::TarReader.new(File.open(pth)) do |tar|
        tar.each do |entry|
          File.open(File.join(dst, entry.full_name), 'w') do |file|
            file.write(entry.read)
          end
        end
      end

      File.delete(pth)
    end

    def destroy
      @container.kill
      @container.delete(:force => true)
    end

  end
end